from itertools import islice


def nth(iterable, n, default=None):
    "Returns the nth item or a default value"
    return next(islice(iterable, n, None), default)


def first_true(iterable, predicate=None, default=None):
    """
    Returns first value for which predicate(item) is true

    If no value is found, returns default.
    """
    return next(filter(predicate, iterable), default)


def true_or_nth(iterable, predicate=None, n=0, default=None):
    """
    Returns first value for which predicate(item) is true

    If no value is found, returns the nth (n) value.

    If the iterable is empty, return default.
    """
    return first_true(iterable, predicate=predicate, default=nth(iterable, n, default=default))



from datetime import datetime
from urllib.parse import urlsplit

from django.template import Library, Node
from django.template.defaultfilters import escape, stringfilter
from django.utils import html, timezone

import bleach
from wagtailgeowidget.app_settings import (
    GEO_WIDGET_ZOOM,
    GOOGLE_MAPS_V3_APIKEY,
)


register = Library()


@register.inclusion_tag('wagtail_extensions/partials/map.html')
def map(location, zoom=GEO_WIDGET_ZOOM):
    return {
        'location': location,
        'zoom': zoom,
    }


@register.inclusion_tag('wagtail_extensions/partials/map_assets.html')
def map_assets():
    return {
        'api_key': GOOGLE_MAPS_V3_APIKEY,
    }


@register.filter
@stringfilter
def humanize_url(value):
    return urlsplit(value)[1]


@register.simple_tag
def humanize_date_range(start, end):
    day_fmt = '%d %b %Y'
    if start == end:
        return start.strftime(day_fmt)
    elif (start.month, start.year) == (end.month, end.year):
        start_fmt = '%d'
    else:
        start_fmt = '%d %b %Y'
    return "{} - {}".format(start.strftime(start_fmt), end.strftime(day_fmt))


@register.filter
def bleachclean(value):
    return bleach.clean(value, strip=True)


@register.simple_tag
def block_method(block_value, method_name):
    method = getattr(block_value.block, method_name)
    if method:
        return method(block_value)
    else:
        return None


@register.inclusion_tag('wagtail_extensions/partials/track_form_submission.html')
def track_form_submission(request):
    submitted = request.session.get('enquiry_form_submitted', False)
    if submitted:
        # Remove the session variable
        del request.session['enquiry_form_submitted']
        try:
            submitted_time = datetime.strptime(submitted, '%Y-%m-%d %H:%M %z')
        except ValueError:
            return {'enquiry_form_submitted': False}

        if (timezone.now() - submitted_time).seconds < 60:
            return {'enquiry_form_submitted': True}

    return {'enquiry_form_submitted': False}


def page_menu_children(page, calling_page=None):
    children = page.get_children().live().in_menu()
    for child in children:
        child.active = (calling_page.url.startswith(child.url) if calling_page else False)
    return children


@register.inclusion_tag('wagtail_extensions/partials/menu.html', takes_context=True)
def menu(context, parent, calling_page=None):
    menuitems = page_menu_children(parent, calling_page)

    for menuitem in menuitems:
        menuitem.children = page_menu_children(menuitem, calling_page)

    return {
        'calling_page': calling_page,
        'menuitems': menuitems,
        # required by the pageurl tag that we want to use within this template
        'request': context['request'],
    }


@register.tag
def metablock(parser, token):
    """
    Remove newlines, excessive whitespace, and HTML tags; and escape the
    content of meta blocks.
    """
    nodelist = parser.parse(('endmetablock',))
    parser.delete_first_token()
    return MetaBlockNode(nodelist)


class MetaBlockNode(Node):
    def __init__(self, nodelist):
        self.nodelist = nodelist

    def render(self, context):
        output = self.nodelist.render(context)
        output = output.replace("\n", "")
        output = " ".join(output.split()).replace(" ,", ",").replace(" .", ".")
        output = html.strip_tags(unescape(output))
        return escape(output)


def unescape(text):
    return text.replace(
        "&amp;", "&").replace("&lt;", "<").replace("&gt;", ">").replace(
        "&quot;", '"').replace("&#39;", "'")



import os
from setuptools import find_packages, setup

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

test_requires = [
    'freezegun>=0.3,<0.4',
    'pytest-django>=3.8,<3.9',
    'pytest-xdist>=1.29,<1.32',
    'psycopg2-binary',
]

setup(
    name='regulus-wagtail-extensions',
    version='2.5.2',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',
    description='A few useful extensions for Wagtail',
    url='https://github.com/regulusweb/wagtail-extensions',
    author='Regulus Ltd',
    author_email='reg@regulusweb.com',
    python_requires='>=3.5',
    install_requires=[
        'wagtail>=2.7,<2.9',
        'django-phonenumber-field',
        'phonenumbers',
        'python-dateutil',
        'wagtailgeowidget',
        # contact
        'django-honeypot>=0.6.0',
        'django-crispy-forms>=1.7',
        'bleach>=2.0.0',
    ],
    extras_require={
        'test': test_requires,
    },
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-13 05:22
import django.contrib.postgres.fields.jsonb
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ContactSubmission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_submitted', models.DateTimeField(auto_now_add=True)),
                ('data', django.contrib.postgres.fields.jsonb.JSONField()),
            ],
            options={
                'ordering': ['-date_submitted'],
            },
        ),
    ]
